#	Copyright (c) Alexander Sedov 2008

#	This file is part of Nodes.
#	
#	Nodes is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	Nodes is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Nodes.  If not, see <http://www.gnu.org/licenses/>.

"""Basic Node types."""
from core import *

#Neuron-likes
class GeneralizedNeuron(Node):
	def __init__(self, inputs, output, iweights=None, oweights=None):
		Node.__init__(self, inputs, outputs, iweights, oweights)

	def init_prepare(self):
		Node.init_prepare(self)

	def check_valid(self):
		assert (self.iN and self.oN), "GeneralizedNeuron shouldn't be null"
		return True

	def calculate(self):
		x=sum(self.input)/self.iN
		self.output+=x #and no sigmoids!

	def calc_drv(self, value):
		return 1/self.iN

class StandardNeuron(GeneralizedNeuron):
	def __init__(self, inputs, output, iweights=None, oweight=1.):
		GeneralizedNeuron.__init__(self, inputs, [output], iweights, [oweight])

	def init_prepare(self):
		GeneralizedNeuron.init_prepare(self)

	def calculate(self):
		x=sum(self.input)/self.iN
		self.output[0]=x

	def calc_drv(self, value):
		return 1/self.iN

class DistributorNeuron(GeneralizedNeuron):
	def __init__(self, input, outputs, iweight=1., oweights=None):
		GeneralizedNeuron.__init__(self, [input], outputs, [iweight], oweights)

	def init_prepare(self):
		GeneralizedNeuron.init_prepare(self)

	def calculate(self):
		self.output+=self.input[0]

#Memory
class InputMemory(Node):
	def __init__(self, outputs, oweights=None, weight=1.):
		Node.__init__(self, [], outputs, [], oweights)
		self.common_weight=weight
		self.value=0

	def set(self, value):
		self.value=value

	def calculate(self):
		self.output=self.output+(self.value*self.common_weight)
		print ':::', self.value, self.common_weight

	def back_propagation(self, targets):
		assert len(targets)==self.oN
		targets=array(targets)
		delta=(targets-self.ovalues)/self.output_weights*self.common_weight
		self.common_weight+=(sum(delta)*self.value)*self.velocity

class OutputMemory(Node):
	def __init__(self, inputs, iweights=None, weight=1.):
		Node.__init__(self, inputs, [], iweights, [])
		self.common_weight=weight
		self.value=0

	def get(self):
		return self.value

	def calculate(self):
		self.value=(sum(self.input)/self.iN)*self.common_weight

	def back_propagation(self, target):
		try:
			target=target[0]
		except:
			pass
		d=(target-self.value)*self.common_weight
		self.input_weights+=(d*self.ivalues)*self.velocity
		map(lambda x, y: x.back_propagate(d*y), self.inputs, self.input_weights)

class AssociativeMemory(Node):
	def __init__(self, key, value, reqin, setin, output, iweight=1., reqweight=1., oweight=1.):
		Node.__init__([reqin, setin], [output], [reqweight, iweight], [oweight])
		self.key=key
		self.value=value
		self.sendit=True

	def calculate(self):
		if self.input[1]:
			self.value=self.input[1]
			self.sendit=False
		elif round(self.input[0])==self.key:
			output[0]=self.value
		else:
			self.sendit=False

	def put_values(self):
		if self.sendit:
			Node.put_values(self)
		else:
			self.sendit=True

	def __getstate_extra__(self):
		return self.key, self.value

	def __setstate_extra__(self, state):
		self.key, self.value=state
		self.sendit=True
		
